//=============================================================================
// Project:    SLP - An Interpreter for Super Logic Programs / Static Semantics
// Module:     rules.h
// Purpose:    List of rules
// Last Change:03.10.2001
// Language:   C++
// Author:     Stefan Brass
// Email:      Stefan.Brass@informatik.uni-giessen.de, sbrass@sis.pitt.edu
// Address:    Universitaet Giessen, Arndtstr. 2, 35392 Giessen, Germany
// Copyright:  (c) 2001 by Stefan Brass
// Licence:    Distributed under the conditions of the GNU Public Licence.
//=============================================================================

//=============================================================================
// Include File Frame:
//=============================================================================

#ifndef	RULES_INCLUDED
#define RULES_INCLUDED

//=============================================================================
// Used Types and Macros:
//=============================================================================

#ifndef VER_INCLUDED
#include "ver.h"
#endif

#ifndef BOOL_INCLUDED
#include "bool.h"
#endif

#ifndef CHECK_INCLUDED
#include "check.h"
#endif

#ifndef RULE_INCLUDED
#include "rule.h"
#endif

//=============================================================================
// Private Constants:
//=============================================================================

//-----------------------------------------------------------------------------
// RULES_MAGIC: Magic number (identifies objects of this class for debugging).
//-----------------------------------------------------------------------------

#ifdef VER_DEBUG
static const long RULES_MAGIC =	0x52555300L;	// 'RUS\0'
#endif

//=============================================================================
// Class for Lists of Terms:
//=============================================================================

class rules_c {

	public:

//-----------------------------------------------------------------------------
// Methods:
//-----------------------------------------------------------------------------

	// Constructor:
	rules_c(rule_t head);

	// Destructor:
	~rules_c(void);

	// Head of the list:
	inline rule_t	get_head(void) const
		{ CHECK(valid(), "rules_c::get_head: Invalid Object");
		  return(Head); }

	// Tail of the list:
	inline rules_c*	get_tail(void) const
		{ CHECK(valid(), "rules_c::get_tail: Invalid Object");
		  return(Tail); }

	// Append a list (only possible if tail empty):
	inline void	append(rules_c* tail)
		{ CHECK(valid(), "rules_c::append: Invalid Object");
		  CHECK(tail != 0, "rules_c::append: List is NIL");
		  CHECK(tail->valid(), "rules_c::append: Tail is invalid");
		  CHECK(Tail == 0, "rules_c::append: Tail already set");
		  Tail = tail; }

	// Print this list of rules:
	void print(void) const;

//-----------------------------------------------------------------------------
// Debugging Support:
//-----------------------------------------------------------------------------

#ifdef	VER_DEBUG
	// Integrity Check:
	public:
	bool_t	valid(void) const;

	// Magic number (identifies objects of this class for debugging).
	private:
	long	Magic;		// Must be "RULES_MAGIC"
#endif

//-----------------------------------------------------------------------------
// Private Members:
//-----------------------------------------------------------------------------

	private:

	// Head: First term in the list.
	rule_t	Head;

	// Tail: Rest of the list (i.e. this list except the first term).
	rules_c *Tail;

//-----------------------------------------------------------------------------
// End of Class:
//-----------------------------------------------------------------------------

};

//-----------------------------------------------------------------------------
// Define pointer type:
//-----------------------------------------------------------------------------

typedef rules_c *rules_t;

//=============================================================================
// End of Include File:
//=============================================================================

#endif

